<?php
/* -----------------------------------------------------------------
Copyright (c) 2025 IT-Recht Kanzlei- https://www.it-recht-kanzlei.de
Released under the MIT License (Expat)
[https://opensource.org/licenses/MIT]
------------------------------------------------------------------ */
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\DBAL\ParameterType;

use ITRechtKanzlei\Gambio\Configuration as GambioConfiguration;

use ITRechtKanzlei\LTIPushData;

/**
 * Class ItrkLegalTextInterfaceModuleCenterModuleController
 * @extends    AbstractModuleCenterModuleController
 * @category   System
 * @package    Modules
 * @subpackage Controllers
 */
class ItrkLegalTextInterfaceModuleCenterModuleController extends AbstractModuleCenterModuleController
{
    const ROUTE_ADMIN = 'admin.php?do=ItrkLegalTextInterfaceModuleCenterModule/';

    protected $subTitle = '';

    /** @var Connection */
    protected $connection = null;

    protected function _init() {
    }

    private function outputPage($view, $tmplData = []) {
        $pageTitle = $this->languageTextManager->get_text('itrklegaltextinterface_title')
            .(!empty($this->subTitle) ? ' - '.$this->subTitle : '');

        $data = MainFactory::create('KeyValueCollection', $tmplData);

        $pluginDir = GambioConfiguration::getPluginDir();

        $assets = MainFactory::create('AssetCollection', [
            MainFactory::create('Asset', DIR_WS_CATALOG.$pluginDir.'Admin/Styles/itrk.css'),
            MainFactory::create('Asset', 'module_center_module.itrklegaltextinterface.lang.inc.php'),
            MainFactory::create('Asset', 'itrklegaltextinterface.lang.inc.php'),
        ]);

        $subNavigation = MainFactory::create('ContentNavigationCollection', []);
        foreach (['Configuration', 'Information'] as $tab) {
            $ltab = strtolower($tab);
            $subNavigation->add(
                new StringType($this->languageTextManager->get_text('tab_'.$ltab, 'itrklegaltextinterface')),
                new StringType(self::ROUTE_ADMIN.$tab),
                new BoolType($ltab === $view)
            );
        }

        return MainFactory::create(
            'AdminLayoutHttpControllerResponse',
            new NonEmptyStringType($pageTitle),
            new ExistingFile(new NonEmptyStringType(
                DIR_FS_CATALOG.$pluginDir.'Admin/Html/itrklti_'.$view.'.html'
            )),
            $data,
            $assets,
            $subNavigation
        );
    }

    protected function getMessageStack(): messageStack {
        isset($GLOBALS['messageStack']) or $GLOBALS['messageStack'] = new messageStack();
        return $GLOBALS['messageStack'];
    }

    protected function getPageToken(): string {
        return isset($_SESSION['coo_page_token']) ? $_SESSION['coo_page_token']->generate_token() : '';
    }

    /**
     * Invokes an action method by the given action name.
     *
     * @param string $actionName Name of action method to call, without 'action'-Suffix.
     *
     * @throws LogicException If no action method of the given name exists.
     * @return HttpControllerResponseInterface Response message.
     */
    protected function _callActionMethod($actionName) {
        if (!(bool)gm_get_conf('MODULE_CENTER_ITRKLEGALTEXTINTERFACE_INSTALLED')) {
            return MainFactory::create('RedirectHttpControllerResponse', 'admin.php?do=ModuleCenter');
        }
        return parent::_callActionMethod($actionName);
    }

    protected function getDbConnection(): Connection {
        if ($this->connection === null) {
            $this->connection = \LegacyDependencyContainer::getInstance()->get(Connection::class);
        }
        return $this->connection;
    }

    protected function getLangMap(): array {
        static $langMap = [];
        if (empty($langMap)) {
            $lang = $this->getDbConnection()
                ->executeQuery('
                      SELECT `languages_id`, `name`, `code`
                        FROM `languages` 
                    ORDER BY `languages_id` ASC
                ')
                ->fetchAllAssociative();
            foreach ($lang as  $row) {
                $langMap[$row['languages_id']] = [
                    'code' => $row['code'],
                    'name' => $row['name']
                ];
            }
        }
        return $langMap;
    }

    protected function getFileList(string $type, string $langCode): array {
        if (!GambioConfiguration::getStoreTextAsFile() || empty($langCode)) {
            return [];
        }
        $files = [];
        foreach (glob(GambioConfiguration::getFileLocation().$type.'_'.$langCode.'*') as $file) {
            $m = [];
            if (!preg_match('/\.([a-z]+)$/', $file, $m)) {
                continue;
            }
            $files[$m[1]] = str_replace(DIR_FS_CATALOG, DIR_WS_CATALOG, $file);
        }
        krsort($files);
        return $files;
    }

    protected function getAvailableTexts(): array {
        $texts = $this->getDbConnection()
            ->executeQuery('
                SELECT `content_group` as `gid`,
                       `content_name` as `name`,
                       `languages_id` as `lid`,
                       `content_version` as `dt`,
                       `content_type` as `type`,
                       `content_status` as `status`
                  FROM `content_manager` 
                 WHERE `content_version` LIKE "ITRK%";
              ORDER BY `languages_id` ASC
            ')
            ->fetchAllAssociative();

        $langMap = $this->getLangMap();

        $availableTexts = [];
        foreach ([
            LTIPushData::DOCTYPE_IMPRINT,
            LTIPushData::DOCTYPE_TERMS_AND_CONDITIONS,
            LTIPushData::DOCTYPE_PRIVACY_POLICY,
            LTIPushData::DOCTYPE_CANCELLATION_POLICY
        ] as $type) {
            $ttype = $this->languageTextManager->get_text('legal_text_type_'.$type, 'itrklegaltextinterface');
            $availableTexts[$ttype] = [];
        }

        foreach ($texts as $row) {
            try {
                $type = GambioConfiguration::mapContentIdToLegalTextType($row['gid']);
            } catch (\InvalidArgumentException $e) {
                continue;
            }
            $row['type'] = $type;
            $row['datetime'] = \DateTime::createFromFormat('\I\T\R\K-Ymd_His', $row['dt']);
            $row['status'] = $row['status'] == '1';
            $row['lname'] = $langMap[$row['lid']]['name'] ?? sprintf(
                $this->languageTextManager->get_text('config_option_language_id', 'itrklegaltextinterface'),
                $row['lid']
            );
            $row['files'] = $this->getFileList($type, $langMap[$row['lid']]['code'] ?? '');
            $transType = $this->languageTextManager->get_text('legal_text_type_'.$type, 'itrklegaltextinterface');
            $availableTexts[$transType][] = $row;
        }
        //echo '<pre>'.print_r($availableTexts, true).'</pre>';
        return $availableTexts;
    }

    protected function getAvailableWithdrawlOptions(): array {
        $stmt = $this->getDbConnection()->prepare('
             SELECT `content_group`, `content_title`, `content_name`
               FROM `content_manager`
              WHERE `languages_id` = :language_id
                    AND `content_position` = "elements_withdrawal"
           ORDER BY `content_group` ASC
        ');
        $stmt->bindValue('language_id', (int)($_SESSION['languages_id'] ?? 1), ParameterType::INTEGER);
        $result = $stmt->executeQuery()->fetchAllAssociative();

        $withdrawlOptions = [];
        foreach ($result as $row) {
            $withdrawlOptions[(int)$row['content_group']] =
                $row['content_title']
                ?? $row['content_name']
                ?? sprintf($this->languageTextManager->get_text('config_option_withdrawl_id', 'itrklegaltextinterface'), $row['content_group']);
        }

        return $withdrawlOptions;
    }

    public function actionDefault() {
        if (GambioConfiguration::isConfigured()) {
            $response = MainFactory::create('RedirectHttpControllerResponse', self::ROUTE_ADMIN.'Configuration');
        } else {
            $response = MainFactory::create('RedirectHttpControllerResponse', self::ROUTE_ADMIN.'Information');
        }
        return $response;
    }

    public function actionInformation() {
        $this->subTitle = $this->languageTextManager->get_text('tab_information', 'itrklegaltextinterface');
        return $this->outputPage('information', []);
    }

    /**
     * @return mixed|RedirectHttpControllerResponse
     * @throws Exception
     */
    public function actionSaveConfiguration()
    {
        $this->_validatePageToken();
        $newConfiguration = $this->_getPostData('conf') ?? [];
        if (
            isset($newConfiguration['use_gtc_in_pdf'])
            || isset($newConfiguration['use_withdrawal_in_pdf'])
        ) {
            GambioConfiguration::setUseConditionsInPdf(
                ($newConfiguration['use_gtc_in_pdf'] ?? '0') === '1'
            );
            GambioConfiguration::setUseWithdrawalInPdf(
                ($newConfiguration['use_withdrawal_in_pdf'] ?? '0') === '1'
            );
            GambioConfiguration::setWithdrawlContentId(
                (int)$newConfiguration['withdrawl_content_id']
            );
            GambioConfiguration::setStoreTextAsFile(
                ($newConfiguration['store_text_as_file'] ?? '0') === '1'
            );
            $this->getMessageStack()->add_session(
                $this->languageTextManager->get_text('config_msg_success_saved', 'itrklegaltextinterface'),
                'success'
            );
        }
        return MainFactory::create('RedirectHttpControllerResponse', self::ROUTE_ADMIN.'Configuration');
    }

    public function actionConfiguration() {
        $this->subTitle = $this->languageTextManager->get_text('tab_configuration', 'itrklegaltextinterface');

        // ATTACH_WITHDRAWAL_INFO_IN_ORDER_CONFIRMATION
        return $this->outputPage('configuration', [
            'save_action'  => self::ROUTE_ADMIN.'SaveConfiguration',
            'page_token'   => $this->getPageToken(),
            'token'   => GambioConfiguration::getToken(),
            'api_url' => HTTP_SERVER . DIR_WS_CATALOG,
            'use_gtc_in_pdf' => GambioConfiguration::useConditionsInPdf(),
            'use_withdrawal_in_pdf' => GambioConfiguration::useWithdrawalInPdf(),
            'store_text_as_file' => GambioConfiguration::getStoreTextAsFile(),
            'withdrawal_content_id' => GambioConfiguration::getWithdrawlContentId(),
            'available_withdrawl_options' => $this->getAvailableWithdrawlOptions(),
            'available_texts' => $this->getAvailableTexts(),
        ]);
    }

}
