<?php
/* -----------------------------------------------------------------
Copyright (c) 2025 IT-Recht Kanzlei- https://www.it-recht-kanzlei.de
Released under the MIT License (Expat)
[https://opensource.org/licenses/MIT]
------------------------------------------------------------------ */
namespace ITRechtKanzlei\Gambio;

use DateTime;
use Doctrine\DBAL\Connection;
use Gambio\Core\Configuration\ConfigurationService as GmConfigurationService;
use Gambio\Core\Configuration\Models\Write\Configuration as GmConfigurationWrite;

use ITRechtKanzlei\LTIPushData;

require_once(__DIR__.'/../itrk-plugin-sdk/sdk/LTIPushData.php');

class Configuration
{
    const PLUGIN_VERSION = '2.0.0';
    const CONF_PREFIX = 'configuration/ITRK_LTI_';

    const CONF_TOKEN = 'TOKEN';
    const CONF_USE_CONDITIONS_IN_PDF = 'USE_CONDITIONS_IN_PDF';
    const CONF_USE_WITHDRAWAL_IN_PDF = 'USE_WITHDRAWAL_IN_PDF';
    const CONF_WITHDRAWAL_CONTENT_ID = 'WITHDRAWAL_CONTENT_ID';
    const CONF_STORE_TEXT_AS_FILE    = 'STORE_TEXT_AS_FILE';

    const CONTENT_GROUP_PRIVACY              = 2;
    const CONTENT_GROUP_TOS                  = 3;
    const CONTENT_GROUP_IMPRINT              = 4;
    const CONTENT_GROUP_PAYMENT_AND_SHIPPING = 3889891;
    const CONTENT_GROUP_WITHDRAWAL_DEFAULT   = 3889896;

    const MAP_TYPE_TO_CONTENT_GROUP = [
        LTIPushData::DOCTYPE_IMPRINT => self::CONTENT_GROUP_IMPRINT,
        LTIPushData::DOCTYPE_TERMS_AND_CONDITIONS => self::CONTENT_GROUP_TOS,
        LTIPushData::DOCTYPE_PRIVACY_POLICY => self::CONTENT_GROUP_PRIVACY,
    ];

    private static $instance = null;

    private $configurationService;

    private static $withdrawlOptionIds = [];

    private function __construct() {
        $this->configurationService = \LegacyDependencyContainer::getInstance()->get(GmConfigurationService::class);
    }

    private function read(string $key): ?string {
        $c = $this->configurationService->find(self::CONF_PREFIX.$key);
        return ($c !== null) ? $c->value() : null;
    }

    private function write(string $key, string $value): void {
        if (class_exists(GmConfigurationWrite::class)) {
            $this->configurationService->save(GmConfigurationWrite::create(self::CONF_PREFIX.$key, (string)$value));
        } else {
            $this->configurationService->save(self::CONF_PREFIX.$key, (string)$value);
        }
    }

    private function delete(string $key): void {
        $this->configurationService->delete(self::CONF_PREFIX.$key);
    }

    private static function gi(): self {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public static function deleteAll(): void {
        self::gi()->delete(self::CONF_TOKEN);
        self::gi()->delete(self::CONF_USE_CONDITIONS_IN_PDF);
        self::gi()->delete(self::CONF_USE_WITHDRAWAL_IN_PDF);
        self::gi()->delete(self::CONF_WITHDRAWAL_CONTENT_ID);
        self::gi()->delete(self::CONF_STORE_TEXT_AS_FILE);
    }

    public static function getPluginDir() {
        $dir = str_replace('\\', '/', dirname(dirname(__DIR__)));
        $reldir = str_replace(['\\', DIR_FS_CATALOG], ['/', ''], $dir);
        if ($dir != $reldir) {
            return $reldir.'/';
        }
        return 'GXModules/ItRechtKanzlei/ItrkLegalTextInterface/';
    }

    public static function getToken(): string {
        $token = self::gi()->read(self::CONF_TOKEN);
        if (empty($token)) {
            require_once(__DIR__.'/../itrk-plugin-sdk/sdk/LTI.php');
            $token = \ITRechtKanzlei\LTI::generateToken();
            self::gi()->write(self::CONF_TOKEN, $token);
        }
        return $token;
    }

    public static function useConditionsInPdf(): bool {
        $state = self::gi()->read(self::CONF_USE_CONDITIONS_IN_PDF);
        if ($state === null) {
            return false;
        }
        return $state === '1';
    }

    public static function setUseConditionsInPdf(bool $state): void {
        self::gi()->write(self::CONF_USE_CONDITIONS_IN_PDF, $state);
    }

    public static function useWithdrawalInPdf(): bool {
        $state = self::gi()->read(self::CONF_USE_WITHDRAWAL_IN_PDF);
        if ($state === null) {
            return false;
        }
        return $state === '1';
    }

    public static function setUseWithdrawalInPdf(bool $state): void {
        self::gi()->write(self::CONF_USE_WITHDRAWAL_IN_PDF, $state);
    }

    public static function useInPdf(string $type): bool {
        $method = [self::class, sprintf('use%sInPdf', ucfirst(strtolower($type)))];
        if (!is_callable($method)) {
            return false;
        }
        return call_user_func($method);
    }

    /**
     * List of withdrawl content ids.
     * @return int[]
     */
    private static function getAvailableWithdrawlOptions(): array {
        if (!empty(self::$withdrawlOptionIds)) {
            return self::$withdrawlOptionIds;
        }
        $stmt = \LegacyDependencyContainer::getInstance()->get(Connection::class)
            ->prepare('
                SELECT DISTINCT `content_group`
                    FROM `content_manager`
                    WHERE `content_position` = "elements_withdrawal"
                ORDER BY `content_group` ASC
            ');
        $result = $stmt->executeQuery()->fetchAllAssociative();
        foreach ($result as $row) {
            self::$withdrawlOptionIds[] = (int)$row['content_group'];
        }
        return self::$withdrawlOptionIds;
    }

    public static function getWithdrawlContentId(): int {
        $cId = (int)self::gi()->read(self::CONF_WITHDRAWAL_CONTENT_ID);
        $contentIds = self::getAvailableWithdrawlOptions();
        if (!in_array($cId, $contentIds)) {
            $cId = self::CONTENT_GROUP_WITHDRAWAL_DEFAULT;
            self::setWithdrawlContentId($cId);
        }
        return $cId;
    }

    public static function setWithdrawlContentId(int $cId): void {
        $contentIds = self::getAvailableWithdrawlOptions();
        if (!in_array($cId, $contentIds)) {
            throw new \InvalidArgumentException('Invalid Content Group Id.');
        }
        self::gi()->write(self::CONF_WITHDRAWAL_CONTENT_ID, $cId);
    }

    public static function getStoreTextAsFile(): bool {
        $state = self::gi()->read(self::CONF_STORE_TEXT_AS_FILE);
        if ($state === null) {
            return false;
        }
        return $state === '1';
    }

    public static function setStoreTextAsFile(bool $state): void {
        self::gi()->write(self::CONF_STORE_TEXT_AS_FILE, $state);
    }

    public static function getFileLocation(): string {
        return DIR_FS_CATALOG . 'media/content/itrk_';
    }

    public static function getPluginVersion() {
        return self::PLUGIN_VERSION;
    }

    public static function mapLegalTextTypeToContentId(string $type): int {
        static $map = null;
        if ($map === null) {
            $map = self::MAP_TYPE_TO_CONTENT_GROUP;
            $map[LTIPushData::DOCTYPE_CAMCELLATION_POLICY] = self::getWithdrawlContentId();
        }
        if (!isset($map[$type])) {
            throw new \InvalidArgumentException('Invalid document type.');
        }
        return $map[$type];
    }

    public static function mapContentIdToLegalTextType(int $cId): string {
        static $map = null;
        if ($map === null) {
            $map = array_flip(self::MAP_TYPE_TO_CONTENT_GROUP);
            $map[self::getWithdrawlContentId()] = LTIPushData::DOCTYPE_CAMCELLATION_POLICY;
        }
        if (!isset($map[$cId])) {
            throw new \InvalidArgumentException('Invalid content group id.');
        }
        return $map[$cId];
    }

    public static function isConfigured(): bool {
        return 
            (self::gi()->read(self::CONF_TOKEN) !== null)
            && (self::gi()->read(self::CONF_USE_CONDITIONS_IN_PDF) !== null)
            && (self::gi()->read(self::CONF_USE_WITHDRAWAL_IN_PDF) !== null)
            && (self::gi()->read(self::CONF_WITHDRAWAL_CONTENT_ID) !== null)
            && (self::gi()->read(self::CONF_STORE_TEXT_AS_FILE) !== null)
        ;
    }

}
