<?php
/* -----------------------------------------------------------------
Copyright (c) 2025 IT-Recht Kanzlei- https://www.it-recht-kanzlei.de
Released under the MIT License (Expat)
[https://opensource.org/licenses/MIT]
------------------------------------------------------------------ */
namespace ITRechtKanzlei\Gambio;

use ITRechtKanzlei\Gambio\Configuration as GambioConfiguration;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\DBAL\ParameterType;
use Gambio\Admin\Modules\Language\Services\LanguageReadService;
use Gambio\Core\Configuration\Services\ConfigurationService;

use Psr\Log\LoggerInterface;
use Gambio\Core\Logging\LoggerBuilder;

use ITRechtKanzlei\LTIPushData;

require_once(__DIR__.'/../itrk-plugin-sdk/sdk/require_all.php');

class LtiHandler extends \ITRechtKanzlei\LTIHandler {
    protected const LOG_FILE = 'it-recht-kanzlei-api';

    private const CONTENT_GROUP_PRIVACY              = 2;
    private const CONTENT_GROUP_TOS                  = 3;
    private const CONTENT_GROUP_IMPRINT              = 4;
    private const CONTENT_GROUP_PAYMENT_AND_SHIPPING = 3889891;
    private const CONTENT_GROUP_WITHDRAWAL           = 3889896;

    private const PDF_TYPES = [
        LTIPushData::DOCTYPE_TERMS_AND_CONDITIONS => 'CONDITIONS',
        LTIPushData::DOCTYPE_CAMCELLATION_POLICY  => 'WITHDRAWAL',
    ];

    /** @var LoggerInterface */
    private $logger;

    /** @var Connection */
    private $connection;
    
    /** @var LanguageReadService */
    private $languageReadService;

    /** @var ConfigurationService */
    private $configurationService;

    /**
     * @param Connection           $connection
     * @param LanguageReadService  $languageReadService
     */
    public function __construct() {
        $this->connection           = \LegacyDependencyContainer::getInstance()->get(Connection::class);
        $this->languageReadService  = \LegacyDependencyContainer::getInstance()->get(LanguageReadService::class);
        $this->configurationService = \LegacyDependencyContainer::getInstance()->get(ConfigurationService::class);
        $loggerBuilder              = \LegacyDependencyContainer::getInstance()->get(LoggerBuilder::class);
        $this->logger               = $loggerBuilder->omitRequestData()->changeNamespace(self::LOG_FILE)->build();
    }

    public function preHandleRequest(): void {
        if (!(bool)gm_get_conf('MODULE_CENTER_ITRKLEGALTEXTINTERFACE_INSTALLED')) {
            throw new \ITRechtKanzlei\LTIError(
                'IT-Recht Kanzlei LegalTextInterface Modul ist nicht installiert.',
                \ITRechtKanzlei\LTIError::CONFIGURATION_INCOMPLETE
            );
        }

        $token = GambioConfiguration::getToken();
        if (empty($token)) {
            throw new \ITRechtKanzlei\LTIError(
                'IT-Recht Kanzlei LegalTextInterface Modul wurde noch nicht vollständig konfiguriert.',
                \ITRechtKanzlei\LTIError::CONFIGURATION_INCOMPLETE
            );
        }
    }

    public function isTokenValid(string $token): bool {
        return GambioConfiguration::getToken() === $token;
    }

    public function handleActionGetVersion(): \ITRechtKanzlei\LTIVersionResult {
        $result = new \ITRechtKanzlei\LTIVersionResult();
        return $result;
    }

    public function handleActionGetAccountList(): \ITRechtKanzlei\LTIAccountListResult {
        $result = new \ITRechtKanzlei\LTIAccountListResult();
        $result->addAccount(0, null, []);
        return $result;
    }

    /**
     * Returns an array of available, configured language codes.
     *
     * @return array
     */
    private function getAvailableLanguages(): array
    {
        static $languageCodes = [];
        if (!empty($languageCodes)) {
            return $languageCodes;
        }
        $allLanguages = $this->languageReadService->getLanguages();
        foreach ($allLanguages as $language) {
            $languageCodes[$language->code()] = $language->id();
        }
        return $languageCodes;
    }

    private function mapLegalTextTypeToContentId(string $type): int {
        static $map = null;
        if ($map === null) {
            $withdrawlCid = (int)gm_get_conf('GM_WITHDRAWAL_CONTENT_ID');
            $map = [
                LTIPushData::DOCTYPE_IMPRINT => self::CONTENT_GROUP_IMPRINT,
                LTIPushData::DOCTYPE_TERMS_AND_CONDITIONS => self::CONTENT_GROUP_TOS,
                LTIPushData::DOCTYPE_PRIVACY_POLICY => self::CONTENT_GROUP_PRIVACY,
                LTIPushData::DOCTYPE_CAMCELLATION_POLICY => $withdrawlCid ?? self::CONTENT_GROUP_WITHDRAWAL,
            ];
        }
        return $map[$type];
    }

    /**
     * @param string $type
     * @param string $headline
     * @param string $text
     * @param int    $languageId
     *
     * @throws DBALException
     */
    private function updateContent(
        string $type,
        string $headline,
        string $text,
        int $languageId
    ): void {
        $sql  = '
             UPDATE content_manager cm 
                SET cm.content_text = :text,
                    %s
                    cm.content_type = "content",
                    cm.content_file = "",
                    cm.content_version = :version
              WHERE cm.content_group = :content_group 
                    AND cm.languages_id = :language_id
        ';
        $sql = sprintf($sql, empty($headline) ? '' : 'cm.content_heading = :headline,');

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue('text', $text);
        $stmt->bindValue('headline', $headline);
        $stmt->bindValue('version', 'ITRK-' . (new \DateTimeImmutable())->format('Ymd_His'));
        $stmt->bindValue('content_group', $this->mapLegalTextTypeToContentId($type), ParameterType::INTEGER);
        $stmt->bindValue('language_id', $languageId, ParameterType::INTEGER);
        $stmt->executeQuery();
    }

    /**
     * Updates legal text for inclusion in PDF invoices.
     */
    private function updatePdf(
        string $type,
        string $headline,
        string $text,
        string $languageCode
    ): void {
        if (!isset(self::PDF_TYPES[$type])) {
            return;
        }
        $pdfType = self::PDF_TYPES[$type];
        if (!GambioConfiguration::useInPdf($pdfType)) {
            return;
        }

        $this->configurationService->saveLanguageDependent(
            'gm_configuration/GM_PDF_'.$pdfType,
            $languageCode,
            $text
        );
        if (!empty($headline)) {
            $this->configurationService->saveLanguageDependent(
                'gm_configuration/GM_PDF_HEADING_'.$pdfType,
                $languageCode,
                $headline
            );
        }
    }

    public function handleActionPush(LTIPushData $data): \ITRechtKanzlei\LTIPushResult {
        $languages = $this->getAvailableLanguages();

        if (!isset($languages[$data->getLanguageIso639_1()])) {
            $availableLanguages = [];
            throw new \ITRechtKanzlei\LTIError(
                sprintf(
                    'Die Sprache (%s) ist im Shop nicht aktiviert. Folgende Sprachen sind im Shop aktiviert: %s.',
                    $data->getLanguageIso639_1(),
                    implode(', ', array_keys($languages))
                ),
                \ITRechtKanzlei\LTIError::SAVE_DOCUMENT_ERROR
            );
        }
        $languageId = $languages[$data->getLanguageIso639_1()];

        try {
            $this->updateContent($data->getType(), $data->getTitle(), $data->getTextHtml(), $languageId);
        } catch (DBALException $e) {
            throw new \ITRechtKanzlei\LTIError(
                $e->getMessage(),
                \ITRechtKanzlei\LTIError::SAVE_DOCUMENT_ERROR
            );
        }

        $this->updatePdf($data->getType(), $data->getTitle(), $data->getText(), $data->getLanguageIso639_1());

        $baseFileLocation = sprintf(
            '%s%s_%s',
            GambioConfiguration::getFileLocation(),
            $data->getType(),
            $data->getLanguageIso639_1()
        );

        if (GambioConfiguration::getStoreTextAsFile()) {
            if (file_put_contents($baseFileLocation.'.txt', $data->getText()) === false) {
                throw new \ITRechtKanzlei\LTIError(
                    'Der Rechtstext konnte nicht gespeichert werden.',
                    \ITRechtKanzlei\LTIError::SAVE_DOCUMENT_ERROR
                );
            }
            if (file_put_contents($baseFileLocation.'.html', $data->getTextHtml()) === false) {
                throw new \ITRechtKanzlei\LTIError(
                    'Der Rechtstext konnte nicht gespeichert werden.',
                    \ITRechtKanzlei\LTIError::SAVE_DOCUMENT_ERROR
                );
            }
            if ($data->hasPdf()) {
                if (file_put_contents($baseFileLocation.'.pdf', $data->getPdf()) === false) {
                    throw new \ITRechtKanzlei\LTIError(
                        'Der Rechtstext konnte nicht gespeichert werden.',
                        \ITRechtKanzlei\LTIError::SAVE_PDF_ERROR
                    );
                }
            }
        }

        $targetUrl = null;
        return new \ITRechtKanzlei\LTIPushResult($targetUrl);
    }

}
